@@ -1,7 +1,5 @@
 #===============================================================================
 #
-# $Id: AuthCookieDBI.pm,v 1.60 2011/03/12 20:14:41 matisse Exp $
-#
 # Apache2::AuthCookieDBI
 #
 # An AuthCookie module backed by a DBI database.
@@ -33,7 +31,7 @@ package Apache2::AuthCookieDBI;
 use strict;
 use warnings;
 use 5.010_000;
-our $VERSION = '2.18';
+our $VERSION = '2.19';
 
 use Apache2::AuthCookie;
 use base qw( Apache2::AuthCookie );
@@ -72,6 +70,7 @@ use constant THIRTY_TWO_CHARACTER_HEX_STRING_REGEX =>
 use constant TRUE             => 1;
 use constant WHITESPACE_REGEX => qr/ \s+ /mx;
 use constant LOG_TYPE_AUTH    => 'authentication';
+use constant LOG_TYPE_AUTHZ   => 'authorization';
 use constant LOG_TYPE_SYSTEM  => 'system';
 use constant LOG_TYPE_TIMEOUT => 'timeout';
 
@@ -83,15 +82,11 @@ use constant LOG_TYPE_TIMEOUT => 'timeout';
 
 Apache2::AuthCookieDBI - An AuthCookie module backed by a DBI database.
 
-=head1 VERSION
-
-    This is version 2.14
-
 =head1 COMPATIBILITY
 
-Starting with version 2.03 the module is in the Apache2::* namespace,
-L<Apache2::AuthCookieDBI>.  For F<mod_perl1> versions
-there is: L<Apache::AuthCookieDBI>
+Starting with version 2.03, this module is in the Apache2::* namespace,
+L<Apache2::AuthCookieDBI>.  For F<mod_perl 1.x> versions,
+there is still L<Apache::AuthCookieDBI>.
 
 =head1 SYNOPSIS
 
@@ -103,7 +98,6 @@ there is: L<Apache::AuthCookieDBI>
     # before all other modules using DBI - just uncomment the next line:
     #PerlModule Apache::DBI  
    
-     
     PerlModule Apache2::AuthCookieDBI
     PerlSetVar WhatEverPath /
     PerlSetVar WhatEverLoginScript /login.pl
@@ -557,7 +551,7 @@ sub _percent_decode {
 # _dbi_connect -- Get a database handle.
 
 sub _dbi_connect {
-    my ( $class, $r ) = @_;
+    my ( $class, $r, $config_hash ) = @_;
     Carp::confess('Failed to pass Apache request object') if not $r;
 
     my ( $pkg, $file, $line, $sub ) = caller(1);
@@ -565,7 +559,7 @@ sub _dbi_connect {
     $class->logger( $r, Apache2::Const::LOG_INFO, $info_message, undef,
         LOG_TYPE_SYSTEM, $r->uri );
 
-    my %c = $class->_dbi_config_vars($r);
+    my %c = $config_hash ? %$config_hash : $class->_dbi_config_vars($r);
 
     my $auth_name = $r->auth_name;
 
@@ -580,7 +574,6 @@ sub _dbi_connect {
         return $dbh;
     }
     else {
-
         my $error_message
             = "${class}\tcouldn't connect to $c{'DBI_DSN'} for auth realm $auth_name";
         $class->logger( $r, Apache2::Const::LOG_ERR, $error_message,
@@ -590,14 +583,15 @@ sub _dbi_connect {
 }
 
 #-------------------------------------------------------------------------------
-# _get_crypted_password -- Get the users' password from the database
-sub _get_crypted_password {
-    my ( $class, $r, $user ) = @_;
-    my $dbh       = $class->_dbi_connect($r) || return;
-    my %c         = $class->_dbi_config_vars($r);
+# _get_crypted_password -- Get the users' password from the database.
+
+  sub _get_crypted_password {
+    my ( $class, $r, $user, $config_hash ) = @_;
+    my %c         = $config_hash ? %$config_hash : $class->_dbi_config_vars($r);
+    my $dbh       = $class->_dbi_connect($r, \%c) || return;
     my $auth_name = $r->auth_name;
 
-    if ( !$class->user_is_active( $r, $user ) ) {
+    if ( !$class->user_is_active( $r, $user, \%c ) ) {
         my $message
             = "${class}\tUser '$user' is not active for auth realm $auth_name.";
         $class->logger( $r, Apache2::Const::LOG_NOTICE, $message, $user,
@@ -619,6 +613,12 @@ sub _get_crypted_password {
       AND $PasswordField IS NOT NULL)
 SQL
     my $sth = $dbh->prepare_cached($sql_query);
+    unless ( defined $sth ) {
+        my $message = "${class}\tcouldn\'t prepare statement handle to $c{'DBI_DSN'} for auth realm $auth_name";
+        $class->logger( $r, Apache2::Const::LOG_ERR, $message, $user,
+                        LOG_TYPE_AUTH, $r->uri );
+        return;
+    }
     $sth->execute($user);
     ($crypted_password) = $sth->fetchrow_array();
     $sth->finish();
@@ -633,6 +633,66 @@ SQL
     return $crypted_password;
 }
 
+#-------------------------------------------------------------------------------
+# _prepare_group_query -- Prepare the database query used to determine whether
+# the authenticated user is a member of a group.
+
+sub _prepare_group_query {
+    my ( $class, $dbh, $config_hash ) = @_;
+
+    # Get the configuration information.
+    my %c = %$config_hash;
+    my $DBI_GroupUserField = $dbh->quote_identifier($c{'DBI_GroupUserField'});
+    my $DBI_GroupsTable = $dbh->quote_identifier($c{'DBI_GroupsTable'});
+    my $DBI_GroupField = $dbh->quote_identifier($c{'DBI_GroupField'});
+
+    my $sth = $dbh->prepare_cached( <<"EOS" );
+SELECT $DBI_GroupUserField
+FROM $DBI_GroupsTable
+WHERE $DBI_GroupField = ?
+AND $DBI_GroupUserField = ?
+EOS
+
+    return $sth;
+}
+
+#-------------------------------------------------------------------------------
+# _check_group_membership -- Query the database to see if the authenticated
+# user is a member of the specified group(s).
+
+sub _check_group_membership {
+    my ( $class, $r, $sth, $groups_ref, $debug ) = @_;
+
+    if ( !defined $debug ) {
+        $debug = $r->dir_config('AuthCookieDebug') || 0;
+    }
+    my $user = $r->user;
+
+    # Loop through all the groups to see if we are a member of any:
+    foreach my $group (@$groups_ref) {
+        $r->server->log_error("${class}\tchecking if user $user is a member of group $group") if ($debug >= 4);
+        $sth->execute( $group, $user );
+        if ( $sth->fetchrow_array ) { # query successful; user is in group
+            $sth->finish();
+            $r->server->log_error("${class}\tauthorized -- user $user is a member of group $group") if ($debug >= 4);
+            # add the group to an ENV var that CGI programs can access:
+            $r->subprocess_env( 'AUTH_COOKIE_DBI_GROUP' => $group );
+            return $group;
+        }
+    }
+    $sth->finish();
+
+    # Log a message similar to mod_authz_user.
+    my $auth_name = $r->auth_name;
+    my $message
+        = "${class}\tuser $user was not a member of any of the required groups @$groups_ref for auth realm $auth_name";
+    $class->logger( $r, Apache2::Const::LOG_INFO, $message, $user,
+        LOG_TYPE_AUTHZ, $r->uri );
+
+    return;
+}
+
+#-------------------------------------------------------------------------------
 sub _get_new_session {
     my $class          = shift;
     my $r              = shift;
@@ -654,9 +714,11 @@ sub _get_new_session {
     return \%session;
 }
 
+#-------------------------------------------------------------------------------
 # Takes a list and returns a list of the same size.
 # Any element in the inputs that is defined is returned unchanged. Elements that
 # were undef are returned as empty strings.
+
 sub _defined_or_empty {
     my @args        = @_;
     my @all_defined = ();
@@ -714,6 +776,7 @@ sub authen_cred {
 
     # get the crypted password from the users database for this user.
     my $crypted_password = $class->_get_crypted_password( $r, $user, \%c );
+    return unless ( defined $crypted_password );
 
     # now return unless the passwords match.
     my $crypt_type = lc $c{'DBI_CryptType'};
@@ -949,56 +1012,30 @@ sub group {
     my ( $class, $r, $groups ) = @_;
     my @groups = split( WHITESPACE_REGEX, $groups );
 
-    my $auth_name = $r->auth_name;
-
     # Get the configuration information.
     my %c = $class->_dbi_config_vars($r);
 
-    my $user = $r->user;
-
     # See if we have a row in the groups table for this user/group.
-    my $dbh = $class->_dbi_connect($r) || return Apache2::Const::SERVER_ERROR;
+    my $dbh = $class->_dbi_connect($r, \%c)
+      || return Apache2::Const::SERVER_ERROR;
+    my $sth = $class->_prepare_group_query($dbh, \%c)
+      || return Apache2::Const::SERVER_ERROR;
 
-    # Now loop through all the groups to see if we're a member of any:
-    my $DBI_GroupUserField = $dbh->quote_identifier($c{'DBI_GroupUserField'});
-    my $DBI_GroupsTable = $dbh->quote_identifier($c{'DBI_GroupsTable'});
-    my $DBI_GroupField = $dbh->quote_identifier($c{'DBI_GroupField'});
-
-    my $sth = $dbh->prepare_cached( <<"EOS" );
-SELECT $DBI_GroupUserField
-FROM $DBI_GroupsTable
-WHERE $DBI_GroupField = ?
-AND $DBI_GroupUserField = ?
-EOS
-    foreach my $group (@groups) {
-        $sth->execute( $group, $user );
-        if ( $sth->fetchrow_array ) {
-            $sth->finish();
-
-            # add the group to an ENV var that CGI programs can access:
-            $r->subprocess_env( 'AUTH_COOKIE_DBI_GROUP' => $group );
-            return Apache2::Const::OK;
-        }
-    }
-    $sth->finish();
-
-    my $message
-        = "${class}\tuser $user was not a member of any of the required groups @groups for auth realm $auth_name";
-    $class->logger( $r, Apache2::Const::LOG_INFO, $message, $user,
-        LOG_TYPE_AUTH, $r->uri );
-    return Apache2::Const::HTTP_FORBIDDEN;
+    return $class->_check_group_membership($r, $sth, \@groups)
+      ? Apache2::Const::OK
+      : Apache2::Const::HTTP_FORBIDDEN;
 }
 
 sub user_is_active {
-    my ( $class, $r, $user ) = @_;
-    my %c                 = $class->_dbi_config_vars($r);
+    my ( $class, $r, $user, $config_hash ) = @_;
+    my %c = $config_hash ? %$config_hash : $class->_dbi_config_vars($r);
     my $active_field_name = $c{'DBI_UserActiveField'};
 
     if ( !$active_field_name ) {
         return TRUE;    # Default is that users are active
     }
 
-    my $dbh = $class->_dbi_connect($r) || return;
+    my $dbh = $class->_dbi_connect($r, \%c) || return;
     my $ActiveFieldName = $dbh->quote_identifier($active_field_name);
     my $DBI_UsersTable = $dbh->quote_identifier($c{'DBI_UsersTable'});
     my $DBI_UserField  = $dbh->quote_identifier($c{'DBI_UserField'});
@@ -1196,9 +1233,9 @@ A minimal CREATE TABLE statement might look like:
 
 =head1 COPYRIGHT
 
- Copyright (C) 2002 SF Interactive.
+ Copyright (C) 2002 SF Interactive
  Copyright (C) 2003-2004 Jacob Davies
- Copyright (C) 2004-2010 Matisse Enzer
+ Copyright (C) 2004-2019 Matisse Enzer
 
 =head1 LICENSE
 
@@ -1222,6 +1259,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
   Incomplete list of additional contributors (alphabetical by first name):
     Carl Gustafsson
     Chad Columbus
+    Edward J Sabol
     Jay Strauss
     Joe Ingersoll
     Keith Lawson
